from django.db import models

class Bank(models.Model):
    name = models.CharField(max_length=200, blank=False, null=False)
    deactivate = models.BooleanField(default=False, blank=False, null=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    client_app = models.JSONField(null=True, blank=True)

    def __str__(self):
        return self.name



import operator
from functools import reduce
from rest_framework import viewsets
from rest_framework.views import APIView
from django.http.response import Http404, HttpResponse
from rest_framework.response import Response

from django.db.models.query_utils import Q

from models.payment import PaymentMethod, PaymentMethodTaxs
from serializers.payments import PaymentMethodSerializer, PaymentMethodTaxsSerializer


class PaymentMethodViewSet(viewsets.ModelViewSet):

    serializer_class = PaymentMethodSerializer
    queryset = PaymentMethod.objects.all()

    def get_queryset(self):
        queryset = super().get_queryset()

        query = Q(deactivate=False)

        if self.action == "list":
            name = str(self.request.query_params.get("name", "")).split(" ")

            if name != "":
                query &= reduce(
                    operator.__and__,
                    (Q(name__unaccent__icontains=word) for word in name),
                )

        return queryset.filter(query).order_by("name")


class ListPaymentMethodTaxs(viewsets.ModelViewSet):

    serializer_class = PaymentMethodTaxsSerializer
    queryset = PaymentMethodTaxs.objects.all()
    
    def get_queryset(self):
        queryset = super().get_queryset()

        query = Q(deactivate=False)

        if self.action == "list":
            tax = str(self.request.query_params.get("tax", "")).split(" ")

            if tax != "":
                query &= reduce(
                    operator.__and__,
                    (Q(tax__icontains=word) for word in tax),
                )

        return queryset.filter(query).order_by("initial_date")
    



import operator
from functools import reduce
from rest_framework import viewsets
from rest_framework.views import APIView

from django.db.models.query_utils import Q

from models.accounts import Account, AccountPermission
from serializers.accounts import AccountSerializer, AccountPermissionSerializer



class AccountViewSet(viewsets.ModelViewSet):

    serializer_class = AccountSerializer
    queryset = Account.objects.all()
    
    def get_queryset(self):
        queryset = super().get_queryset()

        query = Q(deactivate=False)

        if self.action == "list":
            LIST = "list"
            INDICATORS = "indicators"

            name = str(self.request.query_params.get("name", ""))

            use = self.validate_item_in_list(
                item=self.request.query_params.get("use", "list"),
                list=[LIST, INDICATORS],
                message="Invalid",
                required=True,
            )

            if use == INDICATORS:
                queryset = AccountPermission.get_conta(
                    self.request.user.id, self.request.META["HTTP_CLIENTE_APP_ID"]
                )

            if name != "":
                query &= reduce(
                    operator.__and__,
                    (
                        Q(name__unaccent__icontains=word)
                        for word in name.split(" ")
                    ),
                )

        return queryset.filter(query).order_by("name")


from django.db import models

class Category(models.Model):
    name = models.CharField("Nome", max_length=200, blank=False, null=False)
    principal_category = models.ForeignKey(
        "self", on_delete=models.PROTECT, blank=True, null=True
    )
    is_entry = models.BooleanField(default=True, blank=False, null=False)
    is_out = models.BooleanField(default=True, blank=False, null=False)
    deactivate = models.BooleanField(default=False, blank=False, null=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    client_app = models.JSONField(null=True, blank=True)


    def __str__(self):
        return self.name


from argparse import Action
import operator
from functools import reduce
from rest_framework.response import Response
from rest_framework import viewsets
from rest_framework.views import APIView

from django.db.models.query_utils import Q
from finance_sdk_wlc.finance_sdk.utils import validate_item_in_list, validate_natural_number

from enums import Frequency, Type

from models.entry import Entry
from serializers.entry import EntrySerializer

from models.accounts import Account, AccountPermission


class EntryViewSet( viewsets.ModelViewSet):

    serializer_class = EntrySerializer
    queryset = Entry.objects.all()
    

    def get_queryset(self):
        client_app_id = self.request.query_params.get("client_app")
        employee_id = self.request.query_params.get("employee")

        queryset = self.queryset

        query = (
            (
                Q(
                    account_id__in=AccountPermission.get_conta(
                        self.request.user.id, client_app_id
                    ),
                    account__deactivate=False,
                    account__bank__deactivate=False,
                )
                    | Q(account__isnull=True) )
            & 

             (
               Q(payment_method__isnull=False, payment_method__desativada=False)
                | Q(payment_method__isnull=True)
            )
            & (
                Q(category__isnull=False, category__deactivate=False)
                | Q(category__isnull=True)
            )

            (
                Q(person_isnull=False) | Q(person_isnull=True)
            )
        )

        person_id = self.request.query_params.getlist("person_ids", [])

        if person_id:
            query &= Q(person__id__in=person_id)

        if client_app_id:
            query &= Q(client_app__id=client_app_id)

        cost_center_id = self.request.query_params.getlist("cost_center", [])
        
        if cost_center_id:
            query &= Q(cost_center__id__in=cost_center_id)


        if self.action == "list":
            query_params = self.request.query_params

            entry_type = self.validate_item_in_list(
                item=query_params.get("tipo", False),
                lista=dict(Entry.Type.TYPE).keys(),
                message="Entry type invalid",
                message_if_empty="Entry type required",
                required=True,
            )

            initial_date, final_date = self.validar_periodo_datas(
                initial_date=query_params.get("initial_date", False),
                final_date=query_params.get("final_date", False),
            )

            query &= Q(
                entry_type=entry_type, due_date__gte=initial_date, due_date__lte=final_date
            )

            cost_center = validate_natural_number(
                numero=query_params.get("cost_center", False),
                mensagem="Invalid Cost Center",
            )

            if cost_center:
                query &= Q(cost_center_id=cost_center)

            person = query_params.get("person", False)

            if person:
                person = person.split(" ")
                query &= reduce(
                    operator.__and__,
                    (Q(person__name__unaccent__icontains=param) for param in person),
                )

            category = validate_natural_number(
                number=query_params.get("category", False),
                message="Inválid Category",
            )

            if category:
                query &= Q(categoria_id=category)

            account = validate_natural_number(
                number=query_params.get("account", False), message="Invalid Account"
            )

            if account:
                query &= Q(account_id=account)

        return queryset.filter(query).order_by(
            "due_date", "category", "description", "created_at", "id"
        )

    def list(self, request, *args, **kwargs):
        if "values" not in request.query_params:
            return super().list(request, *args, **kwargs)
 
        serializer_data = EntrySerializer(
            {"Entry": self.get_queryset()}, context={"request": request}
        ).data

        return Response(data=serializer_data["valores"])

    def update(self, request, *args, **kwargs):
        partial = kwargs.pop("partial", False)
        instance = self.get_object()
        recalculate_liquid_value = request.data.get("recalculate", False)
        serializer = EntrySerializer(
            instance,
            data=request.data,
            partial=partial,
            context={"request": request, "recalculate": recalculate_liquid_value},
        )
        serializer.is_valid(raise_exception=True)
        self.perform_update(serializer)

        if getattr(instance, "_prefetched_objects_cache", None):
            instance._prefetched_objects_cache = {}

        return Response(serializer.data)

    def destroy(self, request, *args, **kwargs):
        instance = self.get_object()

        if Entry.objects.filter(reference_record=instance).count() > 1:
            return Response(
                {"message": "Payment deletion requires confirmation."},
                ),
              

        instance.reference_record = None
        instance.save()

        self.perform_destroy(instance)

        return Response(status=status.HTTP_204_NO_CONTENT)

    @Action(methods=["GET"], detail=True, url_path="payments")
    def list_payments(self, request, pk=None):
        entry = self.get_object()

        if entry.reference_record:
            payments = Entry.objects.filter(
                reference_record=entry.reference_record
            ).order_by("id")
        else:
            payments = Entry.objects.filter(id=entry.id)

        
        page = self.paginate_queryset(payments)
        serializer = self.get_serializer(page, many=True)

        return self.get_paginated_response(serializer.data)

        

    @Action(methods=["DELETE"], detail=True, url_path="confirm_payment")
    def confirm_payment_deletion(self, request, pk=None):
        entry = self.get_object()

        payments = Entry.objects.filter(reference_record=entry).exclude(
            pk=entry.pk
        )

        for payment in payments:
            self.perform_destroy(payment)

        entry.reference_record = None
        entry.save()

        self.perform_destroy(entry)

        return Response()


from django.db import transaction
from datetime import date, timedelta

from rest_framework import serializers
from finance_sdk.models.entry import Entry
from finance_sdk.models.accounts import AccountPermission

from enums import Frequency, Type, RegistryClassification
from services.entriesService import generate_entries

class EntrySerializer(serializers.ModelSerializer):

    class Meta:
        model = Entry
        fields = [
            "id",
            "description",
            "value",
            "received_value",
            "liquid_value",
            "category",
            "account",
            "cost_center",
            "payment_date",
            "due_date",
            "payment_method",
            "person",
            "entry_type",
            "registry_classification",
            "tag",
            "number_of_payments",
            "current_payment",
            "frequency", 
            "reference_record",
           
        ]

        read_only_fields = [
            "current_payment",
            "reference_record",
            "frequency",
            "liquid_value",
        ]

    @transaction.atomic
    def create(self, validated_data):
        return generate_entries(self.initial_data, validated_data)

    def update(self, instance, validated_data):
        validated_data["number_of_payments"] = instance.number_of_payments
        validated_data["frequency"] = instance.frequency
        validated_data["reference_record"] = instance.reference_record
        value = validated_data.get("value")
        entry_type = (
            validated_data.get("entry_type")
            if validated_data.get("entry_type") is not None
            else instance.entry_type
        )
        if value is not None and entry_type == Entry.entry_type.ENTRY:
            payment_method = (
                validated_data.get("payment_method")
                if validated_data.get("payment_method") is not None
                else instance.payment_method
            )
            if payment_method is not None:
                validated_data["liquid_value"] = value * (payment_method.tax / 100)

        if self.context.get("recalcular") and instance.forma_pagamento is not None:
            validated_data["valor_liquido"] = instance.valor * (
                instance.forma_pagamento.taxa / 100
            )

        return super().update(instance, validated_data)

    def validate_payment_date(self, value):
        DATE_LIMIT = date(2000, 1, 1)

        if DATE_LIMIT > value:
            raise serializers.ValidationError(
                "Date can't be lower then 2000-01-01"
            )

        return value

    def validate_due_date(self, value):
        DATE_LIMIT = date(2000, 1, 1)

        if DATE_LIMIT > value:
            raise serializers.ValidationError(
                "Date can't be lower then 2000-01-01"
            )
        return value

    def validate_value(self, value):
        if not value > 0:
            raise serializers.ValidationError("The value must be greater than 0")

        return value

    def validate_account(self, value):
        if not AccountPermission.objects.filter(
            account=value, employee_id=self.initial_data.get('employee')
        ).exists():
            raise serializers.ValidationError("Invalid Account")

        return value

    def validate_number_of_payments(self, value):
        if not value > 0:
            raise serializers.ValidationError(
                "Number of payments must be greater then 0"
            )

        return value

    
    def validate(self, attrs):
        frequency = attrs.get("frequency", Frequency.NONE)

        if (frequency == Frequency.NONE) and (attrs.get("number_of_payments", 1) > 1):
            raise serializers.ValidationError(
                {
                    "message": "An accounts payable/receivable entry without any recurrence cannot have more than one payment."
                } 
            )

        if (attrs.get("number_of_payments", 1) == 1) and (
            frequency != Frequency.NONE
        ):
            raise serializers.ValidationError(
                {
                    "message": "The number of payments must be greater than 1 in order to have a frequency."
                }  )

        value = attrs.get("value")
        entry_type = attrs.get("entry_type")
        if value and entry_type == Type.ENTRY:
            paid_value = attrs.get("paid_value", 0)
            if value < paid_value:
                raise serializers.ValidationError(
                    {"message": "The Paid value can't be greater than the value."}
                )

        return attrs


from django.db import models

from .bank import Bank

class Account(models.Model):
    name = models.CharField(max_length=200, blank=False, null=False)
    bank = models.ForeignKey(Bank, on_delete=models.PROTECT)
    deactivate = models.BooleanField(default=False, blank=False, null=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    client_app = models.JSONField(null=True, blank=True)

    def __str__(self):
        return self.name + " - " + self.bank.name


class AccountPermission(models.Model):
    account = models.ForeignKey(Account, on_delete=models.CASCADE)
    employee = models.JSONField()
    deactivate = models.BooleanField(default=False, blank=False, null=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    client_app = models.JSONField(null=True, blank=True)

    def __str__(self):
        return "Account: %s | %s" % (self.account, self.employee)

    @staticmethod
    def get_account(user_id, client_app):
        accounts = Account.objects.filter(
            id__in=[
                AccountPermission.objects.filter(
                    employee__user_id=user_id,
                    client_app_id=client_app,
                    deactivate=False,
                    account__bank__deactivate=False,
                ).values_list("account_id")
            ]
        )

        return accounts


from rest_framework import serializers
from models.category import Category

class CategorySerializer(serializers.ModelSerializer):

    class Meta:
        model = Category
        fields = [
            "id",
            "name",
            "principal_category",
            "is_entry",
            "is_out",
            "deactivate",
        ]

    def validate_name(self, value):
        if not (self.instance and value == self.instance.name):
            if Category.objects.filter(
                name=value, deactivate=False, client_app=self.initial_data.get('client_app')).exists():
                raise serializers.ValidationError(
                    "Already exists a category with this name."
                )

            if Category.objects.filter(
                name=value, deactivate=True, client_app=self.initial_data.get('client_app')).exists():
                raise serializers.ValidationError(
                    "Already exists a category with this name."
                )

        return value


from setuptools import setup, find_packages

setup(
    name='finance_sdk',
    version='1.0.9',
    description='SDK for Finance Module',
    author='WLC Soluções',
    author_email='analiciasouza.11@gmail.com',
    url='https://github.com/wlc-solucoes/wlc-sdk-financeiro.git',
    packages=find_packages(include=['finance_sdk', 'finance_sdk.*']),
    install_requires = ["django", "djangorestframework"],
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django' , 
        'Intended Audience :: Developers', 
        'Programming Language :: Python'
     ],
    
)


from django.db import models

class PaymentMethod(models.Model):
    name = models.CharField("Nome", max_length=200, blank=False, null=False)
    tax = models.DecimalField(
        decimal_places=2, max_digits=9, blank=True, null=True, default=0
    )
    deactivate = models.BooleanField(default=False, blank=False, null=False)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    client_app = models.JSONField(null=True, blank=True)


    def __str__(self):
        return self.nome

    def save(self, *args, **kwargs):
        if self.taxa == "":
            self.taxa = 0

        super(PaymentMethod, self).save(*args, **kwargs)


class PaymentMethodTaxs(models.Model):
    initial_payment = models.PositiveIntegerField()
    final_payment = models.PositiveIntegerField()
    tax = models.DecimalField(decimal_places=2, max_digits=5)
    payment_method = models.ForeignKey(PaymentMethod, on_delete=models.PROTECT)
    client_app = models.JSONField(null=True, blank=True)
